@@ -104,6 +104,14 @@ class KernelExpression(ABC): # Abstract
     def reassign_child(self, old_child, new_child): # NOTE: has to return new_child (used by new_tree_with_self_replaced)
         pass
 
+    @abstractmethod
+    def contains_base(self, bt):
+        pass
+
+    @abstractmethod
+    def is_stationary(self):
+        pass
+
     @abstractmethod
     def to_kernel(self):
         pass
@@ -238,6 +246,13 @@ class SumOrProductKE(KernelExpression): # Abstract
     def term_count(self):
         return sum(self.base_terms.values()) + len(self.composite_terms)
 
+    def contains_base(self, bts):
+        if not isinstance(bts, list): bts = [bts]
+        return any([bt in self.base_terms for bt in bts]) or any([ct.contains_base(bts) for ct in self.composite_terms])
+
+    def is_stationary(self):
+        return all([ns not in self.base_terms for ns in non_stationary_kerns]) and all([ct.is_stationary() for ct in self.composite_terms])
+
     @abstractmethod
     def to_kernel(self):
         pass
@@ -461,6 +476,13 @@ class ChangeKE(KernelExpression):
         else: self.right = new_child # NOT A deepcopy! # I.e. elif self.right is old_child
         return new_child # NOTE THIS RETURN VALUE (used by new_tree_with_self_replaced)
 
+    def contains_base(self, bts):
+        if not isinstance(bts, list): bts = [bts]
+        return any([branch in bts if isinstance(branch, str) else branch.contains_base(bts) for branch in (self.left, self.right)])
+
+    def is_stationary(self):
+        return False
+
     def to_kernel(self):
         left_ker = self.left.to_kernel() if isinstance(self.left, KernelExpression) else base_str_to_ker(self.left)
         right_ker = self.right.to_kernel() if isinstance(self.right, KernelExpression) else base_str_to_ker(self.right)
@@ -477,10 +499,10 @@ class ChangeKE(KernelExpression):
         self.parameters[self.CP_or_CW].append({p: param_dict[prefix + p] for p in base_k_param_names[self.CP_or_CW]['parameters']})
         same_type_branches = type(self.left) == type(self.right) and\
                              ((isinstance(self.left, KernelExpression) and self.left.GPy_name == self.right.GPy_name) or self.left == self.right)
-        for child in (('left', self.left), ('right', self.right)):
-            postfix = '_1.' if same_type_branches and child[0] == 'right' else '.'
-            if isinstance(child[1], KernelExpression): child[1].match_up_fit_parameters(param_dict, prefix + child[1].GPy_name + postfix)
-            else: self.parameters[child].append({p: param_dict[prefix + base_k_param_names[child[1]]['name'] + postfix + p] for p in base_k_param_names[child[1]]['parameters']})
+        for branch, kex in (('left', self.left), ('right', self.right)):
+            postfix = '_1.' if same_type_branches and branch == 'right' else '.'
+            if isinstance(kex, KernelExpression): kex.match_up_fit_parameters(param_dict, prefix + kex.GPy_name + postfix)
+            else: self.parameters[(branch, kex)].append({p: param_dict[prefix + base_k_param_names[kex]['name'] + postfix + p] for p in base_k_param_names[kex]['parameters']})
         return self
 
     @staticmethod # This would live in kernelExpressionOperations if it did not need to be used within ChangeKEs
@@ -497,13 +519,13 @@ class ChangeKE(KernelExpression):
 
     def sum_of_prods_form(self):
         new_children = []
-        for child in (('left', self.left), ('right', self.right)):
-            sigmoid_parameters = (change_k_sigmoid_names[self.CP_or_CW][child[0]], self.parameters[self.CP_or_CW][0])
-            if isinstance(child[1], str):
-                leaf_params = [self.parameters[k][0] for k in self.parameters.keys() if isinstance(k, tuple) and k[0] == child[0]][0]
-                new_children.append(ProductKE([]).new_bases_with_parameters([(child[1], leaf_params), sigmoid_parameters]))
+        for branch, kex in (('left', self.left), ('right', self.right)):
+            sigmoid_parameters = (change_k_sigmoid_names[self.CP_or_CW][branch], self.parameters[self.CP_or_CW][0])
+            if isinstance(kex, str):
+                leaf_params = [self.parameters[k][0] for k in self.parameters.keys() if isinstance(k, tuple) and k[0] == branch][0]
+                new_children.append(ProductKE([]).new_bases_with_parameters([(kex, leaf_params), sigmoid_parameters]))
             else:
-                new_child = child[1].sum_of_prods_form()
+                new_child = kex.sum_of_prods_form()
                 if isinstance(new_child, ProductKE): new_child.new_bases_with_parameters(sigmoid_parameters)
                 else: # I.e. SumKE
                     for pt in new_child.composite_terms: pt.new_bases_with_parameters(sigmoid_parameters)
